import re
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import scipy.stats as stats
from scipy.stats import norm
import e3tools.stat_funcs as sf

def calc_ab_metrics(smt, metric_defs, cont_id, trmt_ids, exp_names={}, round_at=3, sample_size_per_metric=True, verbose=False):
    """Calculate per-user metrics from session metrics table
    * Caveat: sample_size should be additive 
    - i.e. summing up sample size within a group should give the sample size for the group  
    """
    c = cont_id
    smt_res = []
    tbl = smt.groupby(["exp_id"]).agg("sum")
    for m in metric_defs:
        for e in trmt_ids:
            if sample_size_per_metric:
                ss_c, ss_e = tbl.loc[c][m[0]+'_sample_size'], tbl.loc[e][m[0]+'_sample_size']
            else:
                ss_c, ss_e = tbl.loc[c]['sample_size'], tbl.loc[e]['sample_size']
            res = sf.calc_diff_population_means_large_independent_samples(
                ss_c, tbl.loc[c][m[0]+'_sum'], tbl.loc[c][m[0]+'_ssq'],
                ss_e, tbl.loc[e][m[0]+'_sum'], tbl.loc[e][m[0]+'_ssq'])
            res['control'] = exp_names.get(c, c)
            res['treatment'] = exp_names.get(e, e)
            res['metric'] = m[0]
            res['avg_cont'] = tbl.loc[c][m[0]+'_avg']
            res['avg_trmt'] = tbl.loc[e][m[0]+'_avg']
            res['avg_cont'] = (tbl.loc[c][m[0]+'_sum'] / ss_c)
            res['avg_trmt'] = (tbl.loc[e][m[0]+'_sum'] / ss_e)
            if verbose:
                res['abs_diff'] = res['avg_trmt'] - res['avg_cont']
            res['pct_diff'] = (res['avg_trmt'] - res['avg_cont']) / res['avg_cont']
            res['p_value'] = res['P_VALUE_RELATIVE_DIFFERENCE']
            if verbose:
                res['diff_lo'] = res['DIFFERENCE_ERROR_MARGIN'][0]
                res['diff_hi'] = res['DIFFERENCE_ERROR_MARGIN'][1]
            res['n_cont'] = ss_c
            res['n_trmt'] = ss_e
            del(res['P_VALUE_RELATIVE_DIFFERENCE'])
            del(res['P_VALUE'])
            del(res['DIFFERENCE'])
            del(res['DIFFERENCE_ERROR_MARGIN'])
            del(res['CONFIDENCE'])
            smt_res.append(res)
    return pd.DataFrame(smt_res).round(round_at).\
        style.bar(subset=['pct_diff'], align='mid', color=['#d65f5f', '#5fba7d']).format({'pct_diff': "{:.2%}"})

